/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces;

import java.net.URI;
import java.net.URISyntaxException;

import org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces.util.DocumentContext;
import org.bluestemsoftware.open.eoa.ext.idl.wsdl.wsdl20.xerces.util.WSDLParser;
import org.bluestemsoftware.specification.eoa.ext.ExtensionFactory;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.mex.WSMexFeature;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.mex.WSMexFeature.MetadataType;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.WSDLException;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ImportElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.WSDL20Document;
import org.bluestemsoftware.specification.eoa.system.SystemContext;
import org.bluestemsoftware.specification.eoa.system.server.Server;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

public class ImportElementImpl extends AbstractWSDLElement implements ImportElement {

    private static final long serialVersionUID = 1L;
    private WSDL20Document document;

    public ImportElementImpl(WSDL20DocumentImpl owner) {
        super(owner, NAME.getNamespaceURI(), NAME.getPrefix() + ":" + NAME.getLocalPart());
    }

    public ImportElementImpl() {
        super();
    }

    public void setNamespace(String namespace) {
        setAttribute("namespace", namespace);
    }

    public String getNamespace() {
        return getAttribute("namespace").equals("") ? null : getAttribute("namespace");
    }

    public void setLocation(String location) {
        setAttribute("location", location);
    }

    public String getLocation() {
        return getAttribute("location").equals("") ? null : getAttribute("location");
    }

    public WSDL20Document getDocument() throws WSDLException {

        if (document != null) {
            return document;
        }

        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();

        try {
            
            DocumentContext documentContext = ((WSDL20DocumentImpl)ownerDocument).getDocumentContext();
            ExtensionFactory factory = documentContext.getFactory();
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());

            // location attrib may be null in which case it is assumed that
            // referenced document is defined within a separate deployment

            if (getLocation() == null) {
                return null;
            }

            String documentBase = ((WSDL20DocumentImpl)ownerDocument).getBaseURI();
            URI expandedLocation;
            try {
                expandedLocation = null;
                if (documentBase == null) {
                    expandedLocation = new URI(getLocation());
                } else {
                    expandedLocation = new URI(documentBase).resolve(getLocation());
                }
            } catch (URISyntaxException ue) {
                throw new WSDLException("Error resolving imported document." + ue);
            }

            EntityResolver entityResolver = documentContext.getEntityResolver();
            
            // if optional feature, ws-mex is enabled, we run the request through it in
            // case policy is resolved via soap request/response. otherwise retrieve 
            // deployment's entity resolver

            Server server = SystemContext.getContext().getSystem().getServer();
            WSMexFeature metadataFeature = server.getFeature(WSMexFeature.class);

            InputSource inputSource = null;
            if (metadataFeature != null && expandedLocation.isAbsolute()) {
                try {
                    inputSource = metadataFeature.getMetadata(MetadataType.WSDL, expandedLocation.toString());
                } catch (Exception ex) {
                    throw new WSDLException("Error attempting to resolve imported document at location '"
                            + expandedLocation.toString()
                            + "'. "
                            + ex);
                }
            } else {
                try {
                    inputSource = entityResolver.resolveEntity(null, expandedLocation.toString());
                } catch (Exception ex) {
                    throw new WSDLException("Error attempting to resolve imported document at location '"
                            + expandedLocation
                            + "'. "
                            + ex);
                }
            }

            try {
                WSDLParser wsdlParser = new WSDLParser(entityResolver);
                wsdlParser.parse(inputSource);
                document = (WSDL20DocumentImpl)wsdlParser.getDocument();
            } catch (Exception ex) {
                throw new WSDLException("Error attempting to resolve imported document at location '"
                        + inputSource.getSystemId()
                        + "'. "
                        + ex);
            }

            String expectedNamespace = getNamespace();

            if (!document.getDescriptionElement().getTargetNamespace().equals(expectedNamespace)) {
                throw new WSDLException("Error attempting to resolve imported document at location '"
                        + inputSource.getSystemId()
                        + "'. Expected targetNamespace '"
                        + expectedNamespace
                        + "'. Found '"
                        + document.getDescriptionElement().getTargetNamespace()
                        + "'.");
            }

            return document;

        } finally {
            thread.setContextClassLoader(cl);
        }

    }

    // *********************** Identity Methods ************************************
    
    @Override
    public final int hashCode() {
        return hashCode(getNamespace()) + hashCode(getLocation());
    }

    @Override
    public final boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }

        if (!(obj instanceof ImportElement)) {
            return false;
        }

        if ((getNamespace() == ((ImportElement)obj).getNamespace() || (getNamespace() != null && getNamespace()
                .equals(((ImportElement)obj).getNamespace())))
                && (getLocation() == ((ImportElement)obj).getLocation() || (getLocation() != null && getLocation()
                        .equals(((ImportElement)obj).getLocation())))) {
            return true;
        }

        return false;
    }

    private int hashCode(Object object) {
        return (object == null ? 0 : object.hashCode());
    }

}
